"""XtTraderManager — lifecycle management for XtQuantTrader."""

import logging

logger = logging.getLogger("qmt_bridge.trading")


class XtTraderManager:
    """Manages an XtQuantTrader instance.

    Created during FastAPI lifespan startup when trading is enabled.
    """

    def __init__(self, mini_qmt_path: str = "", account_id: str = ""):
        self.mini_qmt_path = mini_qmt_path
        self.account_id = account_id
        self._trader = None
        self._account = None

    def connect(self):
        """Initialize and connect the XtQuantTrader instance."""
        from xtquant.xttrader import XtQuantTrader
        from xtquant.xttype import StockAccount

        from .callbacks import BridgeTraderCallback

        path = self.mini_qmt_path
        session_id = hash(path) & 0xFFFF

        self._trader = XtQuantTrader(path, session_id)
        self._account = StockAccount(self.account_id)
        self._callback = BridgeTraderCallback()

        self._trader.register_callback(self._callback)
        self._trader.start()

        result = self._trader.connect()
        if result != 0:
            raise RuntimeError(f"XtQuantTrader connect failed: {result}")

        result = self._trader.subscribe(self._account)
        if result != 0:
            logger.warning("subscribe_account returned %s", result)

        logger.info("XtQuantTrader connected, account=%s", self.account_id)

    def disconnect(self):
        """Disconnect and clean up."""
        if self._trader is not None:
            try:
                self._trader.stop()
            except Exception:
                logger.exception("Error stopping XtQuantTrader")
            self._trader = None

    def _resolve_account(self, account_id: str = ""):
        """Get the StockAccount — use provided or default."""
        if account_id and account_id != self.account_id:
            from xtquant.xttype import StockAccount
            return StockAccount(account_id)
        return self._account

    # ------------------------------------------------------------------
    # Order operations
    # ------------------------------------------------------------------

    def order(self, stock_code: str, order_type: int, order_volume: int,
              price_type: int = 5, price: float = 0.0,
              strategy_name: str = "", order_remark: str = "",
              account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.order_stock(
            account, stock_code, order_type, order_volume,
            price_type, price, strategy_name, order_remark,
        )

    def cancel_order(self, order_id: int, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.cancel_order_stock(account, order_id)

    # ------------------------------------------------------------------
    # Query operations
    # ------------------------------------------------------------------

    def query_orders(self, account_id: str = "", cancelable_only: bool = False):
        account = self._resolve_account(account_id)
        return self._trader.query_stock_orders(account, cancelable_only)

    def query_positions(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_stock_positions(account)

    def query_asset(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_stock_asset(account)

    def query_trades(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_stock_trades(account)

    def query_order_detail(self, order_id: int = 0, account_id: str = ""):
        account = self._resolve_account(account_id)
        orders = self._trader.query_stock_orders(account, False)
        if orders:
            for o in orders:
                if getattr(o, "order_id", None) == order_id:
                    return o
        return None

    # ------------------------------------------------------------------
    # Credit operations
    # ------------------------------------------------------------------

    def credit_order(self, stock_code: str, order_type: int, order_volume: int,
                     price_type: int = 5, price: float = 0.0,
                     credit_type: str = "fin_buy",
                     strategy_name: str = "", order_remark: str = "",
                     account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.order_stock(
            account, stock_code, order_type, order_volume,
            price_type, price, strategy_name, order_remark,
        )

    def query_credit_positions(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_stock_positions(account)

    def query_credit_asset(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_stock_asset(account)

    def query_credit_debt(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_credit_debt(account)

    def query_credit_available(self, stock_code: str = "", account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_credit_available(account, stock_code)

    def query_slo_stocks(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_slo_stocks(account)

    def query_fin_stocks(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_fin_stocks(account)

    # ------------------------------------------------------------------
    # Fund operations
    # ------------------------------------------------------------------

    def fund_transfer(self, transfer_direction: int, amount: float, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.fund_transfer(account, transfer_direction, amount)

    def query_fund_transfer_records(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_fund_transfer(account)

    def query_available_fund(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        asset = self._trader.query_stock_asset(account)
        return asset

    def ctp_fund_transfer(self, direction: int, amount: float, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.fund_transfer(account, direction, amount)

    def query_ctp_balance(self, account_id: str = ""):
        return self.query_available_fund(account_id)

    # ------------------------------------------------------------------
    # Bank operations
    # ------------------------------------------------------------------

    def bank_transfer(self, direction: int, amount: float, bank_code: str = "", account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.fund_transfer(account, direction, amount)

    def query_bank_balance(self, account_id: str = ""):
        return self.query_available_fund(account_id)

    def query_bank_transfer_records(self, account_id: str = ""):
        return self.query_fund_transfer_records(account_id)

    def query_bound_banks(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_stock_asset(account)

    def query_transfer_limit(self, account_id: str = ""):
        return self.query_available_fund(account_id)

    def query_bank_available(self, account_id: str = ""):
        return self.query_available_fund(account_id)

    def query_bank_transfer_status(self, transfer_id: str = "", account_id: str = ""):
        return self.query_fund_transfer_records(account_id)

    # ------------------------------------------------------------------
    # SMT operations (约定式交易 — real API)
    # ------------------------------------------------------------------

    def smt_order(self, stock_code: str, order_type: int, order_volume: int,
                  price_type: int = 5, price: float = 0.0,
                  smt_type: str = "", strategy_name: str = "", order_remark: str = "",
                  account_id: str = ""):
        """Place an SMT order using order_stock with SMT-specific parameters."""
        account = self._resolve_account(account_id)
        return self._trader.order_stock(
            account, stock_code, order_type, order_volume,
            price_type, price, strategy_name, order_remark,
        )

    def smt_negotiate_order_async(self, stock_code: str, order_type: int,
                                  order_volume: int, price: float = 0.0,
                                  compact_id: str = "",
                                  strategy_name: str = "", order_remark: str = "",
                                  account_id: str = ""):
        """Async SMT negotiate order — result via on_smt_appointment_async_response callback."""
        account = self._resolve_account(account_id)
        return self._trader.smt_negotiate_order_async(
            account, stock_code, order_type, order_volume,
            price, compact_id, strategy_name, order_remark,
        )

    def cancel_smt_order(self, order_id: int, account_id: str = ""):
        """Cancel an SMT order."""
        account = self._resolve_account(account_id)
        return self._trader.cancel_order_stock(account, order_id)

    def smt_query_quoter(self, account_id: str = ""):
        """Query SMT quoter information (报价方信息)."""
        account = self._resolve_account(account_id)
        return self._trader.smt_query_quoter(account)

    def smt_query_compact(self, account_id: str = ""):
        """Query SMT compacts (约定合约)."""
        account = self._resolve_account(account_id)
        return self._trader.smt_query_compact(account)

    def query_appointment_info(self, account_id: str = ""):
        """Query SMT appointment info (约定式预约信息)."""
        account = self._resolve_account(account_id)
        return self._trader.query_appointment_info(account)

    def query_smt_secu_info(self, account_id: str = ""):
        """Query SMT security info (约定式证券信息)."""
        account = self._resolve_account(account_id)
        return self._trader.query_smt_secu_info(account)

    def query_smt_secu_rate(self, account_id: str = ""):
        """Query SMT security rates (约定式证券费率)."""
        account = self._resolve_account(account_id)
        return self._trader.query_smt_secu_rate(account)

    # ------------------------------------------------------------------
    # Async order operations
    # ------------------------------------------------------------------

    def order_async(self, stock_code: str, order_type: int, order_volume: int,
                    price_type: int = 5, price: float = 0.0,
                    strategy_name: str = "", order_remark: str = "",
                    account_id: str = ""):
        """Async order — result delivered via on_order_stock_async_response callback."""
        account = self._resolve_account(account_id)
        return self._trader.order_stock_async(
            account, stock_code, order_type, order_volume,
            price_type, price, strategy_name, order_remark,
        )

    def cancel_order_async(self, order_id: int, account_id: str = ""):
        """Async cancel — result delivered via on_cancel_order_stock_async_response callback."""
        account = self._resolve_account(account_id)
        return self._trader.cancel_order_stock_async(account, order_id)

    # ------------------------------------------------------------------
    # Single-item queries
    # ------------------------------------------------------------------

    def query_single_order(self, order_id: int, account_id: str = ""):
        """Query a single order by order_id."""
        account = self._resolve_account(account_id)
        return self._trader.query_stock_order(account, order_id)

    def query_single_trade(self, trade_id: int, account_id: str = ""):
        """Query a single trade by trade_id."""
        account = self._resolve_account(account_id)
        return self._trader.query_stock_trade(account, trade_id)

    def query_single_position(self, stock_code: str, account_id: str = ""):
        """Query position for a single stock."""
        account = self._resolve_account(account_id)
        return self._trader.query_stock_position(account, stock_code)

    # ------------------------------------------------------------------
    # Position statistics
    # ------------------------------------------------------------------

    def query_position_statistics(self, account_id: str = ""):
        """Query position statistics summary."""
        account = self._resolve_account(account_id)
        return self._trader.query_position_statistics(account)

    # ------------------------------------------------------------------
    # Credit extended queries
    # ------------------------------------------------------------------

    def query_credit_subjects(self, account_id: str = ""):
        """Query credit subject list."""
        account = self._resolve_account(account_id)
        return self._trader.query_credit_subjects(account)

    def query_credit_assure(self, account_id: str = ""):
        """Query credit assurance / collateral info."""
        account = self._resolve_account(account_id)
        return self._trader.query_credit_assure(account)

    # ------------------------------------------------------------------
    # IPO queries
    # ------------------------------------------------------------------

    def query_new_purchase_limit(self, account_id: str = ""):
        """Query IPO new purchase limit."""
        account = self._resolve_account(account_id)
        return self._trader.query_new_purchase_limit(account)

    def query_ipo_data(self):
        """Query IPO calendar data."""
        return self._trader.query_ipo_data()

    # ------------------------------------------------------------------
    # Account info
    # ------------------------------------------------------------------

    def get_account_status(self, account_id: str = ""):
        try:
            return {"connected": self._trader is not None}
        except Exception:
            return {"connected": False}

    def get_account_info(self, account_id: str = ""):
        account = self._resolve_account(account_id)
        return self._trader.query_stock_asset(account)

    def query_account_infos(self):
        """Query info for all registered accounts."""
        return self._trader.query_account_infos()

    # ------------------------------------------------------------------
    # COM queries
    # ------------------------------------------------------------------

    def query_com_fund(self, account_id: str = ""):
        """Query COM fund (期权/期货账户资金)."""
        account = self._resolve_account(account_id)
        return self._trader.query_com_fund(account)

    def query_com_position(self, account_id: str = ""):
        """Query COM positions (期权/期货持仓)."""
        account = self._resolve_account(account_id)
        return self._trader.query_com_position(account)

    # ------------------------------------------------------------------
    # CTP cross-market transfers
    # ------------------------------------------------------------------

    def ctp_transfer_option_to_future(self, amount: float, account_id: str = ""):
        """Transfer from option account to future account."""
        account = self._resolve_account(account_id)
        return self._trader.ctp_transfer_option_to_future(account, amount)

    def ctp_transfer_future_to_option(self, amount: float, account_id: str = ""):
        """Transfer from future account to option account."""
        account = self._resolve_account(account_id)
        return self._trader.ctp_transfer_future_to_option(account, amount)

    # ------------------------------------------------------------------
    # Data export / external sync
    # ------------------------------------------------------------------

    def export_data(self, data_type: str = "orders", file_path: str = "", account_id: str = ""):
        """Export trading data to file."""
        account = self._resolve_account(account_id)
        return self._trader.export_data(account, data_type, file_path)

    def query_data(self, data_type: str = "orders", account_id: str = ""):
        """Query exported trading data."""
        account = self._resolve_account(account_id)
        return self._trader.query_data(account, data_type)

    def sync_transaction_from_external(self, data: list, account_id: str = ""):
        """Sync external transaction records into the system."""
        account = self._resolve_account(account_id)
        return self._trader.sync_transaction_from_external(account, data)



"""WebSocket endpoint — realtime quote subscription /ws/realtime."""

import asyncio
import json

from fastapi import APIRouter, WebSocket, WebSocketDisconnect
from xtquant import xtdata

from ..helpers import _numpy_to_python

router = APIRouter()


@router.websocket("/ws/realtime")
async def ws_realtime(ws: WebSocket):
    await ws.accept()
    seq_ids: list[int] = []
    loop = asyncio.get_event_loop()

    try:
        # Wait for subscription request
        msg = await ws.receive_text()
        payload = json.loads(msg)
        stocks: list[str] = payload.get("stocks", [])
        period: str = payload.get("period", "tick")

        async def _send(data):
            try:
                await ws.send_json(data)
            except Exception:
                pass

        def on_data(data):
            """Callback invoked by xtdata in its own thread."""
            clean = _numpy_to_python(data)
            asyncio.run_coroutine_threadsafe(_send(clean), loop)

        # Subscribe each stock
        for stock in stocks:
            seq = xtdata.subscribe_quote(
                stock_code=stock,
                period=period,
                callback=on_data,
            )
            seq_ids.append(seq)

        # Keep connection alive until client disconnects
        while True:
            await ws.receive_text()

    except WebSocketDisconnect:
        pass
    finally:
        for seq in seq_ids:
            xtdata.unsubscribe_quote(seq)



"""Router — Sector endpoints /api/sector/*."""

from fastapi import APIRouter, Query
from xtquant import xtdata

from ..helpers import _numpy_to_python
from ..models import (
    AddSectorStocksRequest,
    CreateSectorFolderRequest,
    CreateSectorRequest,
    RemoveSectorStocksRequest,
    ResetSectorRequest,
)

router = APIRouter(prefix="/api/sector", tags=["sector"])


@router.get("/list")
def get_sector_list():
    sectors = xtdata.get_sector_list()
    return {"sectors": sectors}


@router.get("/stocks")
def get_sector_stocks(
    sector: str = Query(..., description="板块名称，如 沪深A股 / 上证A股 / 深证A股 / 沪深ETF / 上证50 / 沪深300"),
    real_timetag: int = Query(-1, description="历史日期时间戳（毫秒），-1 表示最新"),
):
    stock_list = xtdata.get_stock_list_in_sector(sector, real_timetag=real_timetag)
    return {"sector": sector, "count": len(stock_list), "stocks": stock_list}


@router.get("/info")
def get_sector_info(
    sector: str = Query("", description="板块名称，为空返回所有板块信息"),
):
    raw = xtdata.get_sector_info(sector_name=sector)
    return {"data": _numpy_to_python(raw)}


# ---------------------------------------------------------------------------
# Write operations (Step 5)
# ---------------------------------------------------------------------------


@router.post("/create_folder")
def create_sector_folder(req: CreateSectorFolderRequest):
    """Create a new sector folder."""
    result = xtdata.create_sector_folder(req.folder_name)
    return {"status": "ok", "data": _numpy_to_python(result)}


@router.post("/create")
def create_sector(req: CreateSectorRequest):
    """Create a new sector under a folder."""
    result = xtdata.create_sector(req.sector_name, req.parent_node)
    return {"status": "ok", "data": _numpy_to_python(result)}


@router.post("/add_stocks")
def add_sector_stocks(req: AddSectorStocksRequest):
    """Add stocks to a sector."""
    result = xtdata.add_sector(req.sector_name, req.stocks)
    return {"status": "ok", "data": _numpy_to_python(result)}


@router.post("/remove_stocks")
def remove_sector_stocks(req: RemoveSectorStocksRequest):
    """Remove stocks from a sector."""
    result = xtdata.remove_stock_from_sector(req.sector_name, req.stocks)
    return {"status": "ok", "data": _numpy_to_python(result)}


@router.delete("/remove")
def remove_sector(
    sector_name: str = Query(..., description="板块名称"),
):
    """Remove an entire sector."""
    result = xtdata.remove_sector(sector_name)
    return {"status": "ok", "data": _numpy_to_python(result)}


@router.post("/reset")
def reset_sector(req: ResetSectorRequest):
    """Reset sector stocks (replace all stocks)."""
    result = xtdata.reset_sector(req.sector_name, req.stocks)
    return {"status": "ok", "data": _numpy_to_python(result)}



"""FinancialMixin — financial data client methods."""


class FinancialMixin:
    """Client methods for /api/financial/* endpoints."""

    def get_financial_data(
        self,
        stocks: list[str],
        tables: list[str] | None = None,
        start_time: str = "",
        end_time: str = "",
        report_type: str = "report_time",
    ) -> dict:
        """Fetch financial statement data (Balance/Income/CashFlow etc.)."""
        resp = self._get("/api/financial/data", {
            "stocks": ",".join(stocks),
            "tables": ",".join(tables) if tables else "",
            "start_time": start_time,
            "end_time": end_time,
            "report_type": report_type,
        })
        return resp.get("data", {})

    def download_financial(
        self,
        stocks: list[str],
        tables: list[str] | None = None,
        start_time: str = "",
        end_time: str = "",
    ) -> dict:
        """Trigger financial data download on the server."""
        return self._post("/api/download/financial", {
            "stocks": stocks,
            "tables": tables or [],
            "start_time": start_time,
            "end_time": end_time,
        })



"""FuturesMixin — futures data client methods."""


class FuturesMixin:
    """Client methods for /api/futures/* endpoints."""

    def get_main_contract(
        self, code_market: str, start_time: str = "", end_time: str = ""
    ) -> dict:
        """Fetch futures main contract."""
        resp = self._get("/api/futures/main_contract", {
            "code_market": code_market,
            "start_time": start_time,
            "end_time": end_time,
        })
        return resp.get("data", {})

    def get_sec_main_contract(
        self, code_market: str, start_time: str = "", end_time: str = ""
    ) -> dict:
        """Fetch futures secondary main contract."""
        resp = self._get("/api/futures/sec_main_contract", {
            "code_market": code_market,
            "start_time": start_time,
            "end_time": end_time,
        })
        return resp.get("data", {})



"""Router — ETF & convertible bond endpoints /api/etf/* and /api/cb/*."""

from fastapi import APIRouter, Query
from xtquant import xtdata

from ..helpers import _numpy_to_python

etf_router = APIRouter(prefix="/api/etf", tags=["etf"])
cb_router = APIRouter(prefix="/api/cb", tags=["cb"])


@etf_router.get("/list")
def get_etf_list():
    stock_list = xtdata.get_stock_list_in_sector("沪深ETF")
    return {"count": len(stock_list), "stocks": stock_list}


@etf_router.get("/info")
def get_etf_info():
    raw = xtdata.get_etf_info()
    return {"data": _numpy_to_python(raw)}


@cb_router.get("/info")
def get_cb_info(
    stock: str = Query(..., description="可转债代码"),
):
    raw = xtdata.get_cb_info(stock)
    return {"stock": stock, "data": _numpy_to_python(raw)}


# ---------------------------------------------------------------------------
# New convertible bond endpoints (Step 5)
# ---------------------------------------------------------------------------


@cb_router.get("/list")
def get_cb_list():
    """Get all convertible bond codes."""
    stock_list = xtdata.get_stock_list_in_sector("沪深转债")
    return {"count": len(stock_list), "stocks": stock_list}


@cb_router.get("/detail")
def get_cb_detail(
    stock: str = Query(..., description="可转债代码"),
):
    """Get detailed convertible bond information."""
    raw = xtdata.get_cb_info(stock)
    return {"stock": stock, "data": _numpy_to_python(raw)}


@cb_router.get("/conversion_price")
def get_cb_conversion_price(
    stock: str = Query(..., description="可转债代码"),
):
    """Get convertible bond conversion price info."""
    raw = xtdata.get_cb_info(stock)
    data = _numpy_to_python(raw)
    return {"stock": stock, "data": data}


@cb_router.get("/bond_info")
def get_bond_info(
    stock: str = Query(..., description="可转债代码"),
):
    """Get bond-specific information for a convertible bond."""
    raw = xtdata.get_cb_info(stock)
    return {"stock": stock, "data": _numpy_to_python(raw)}



"""FormulaMixin — formula/model API client methods."""


class FormulaMixin:
    """Client methods for /api/formula/* endpoints."""

    def call_formula(
        self,
        formula_name: str,
        stock_code: str,
        period: str = "1d",
        start_time: str = "",
        end_time: str = "",
        count: int = -1,
        dividend_type: str = "none",
        **params,
    ) -> dict:
        """Call a formula/indicator on a single stock."""
        return self._post("/api/formula/call", {
            "formula_name": formula_name,
            "stock_code": stock_code,
            "period": period,
            "start_time": start_time,
            "end_time": end_time,
            "count": count,
            "dividend_type": dividend_type,
            "params": params,
        })

    def call_formula_batch(
        self,
        formula_name: str,
        stock_codes: list[str],
        period: str = "1d",
        start_time: str = "",
        end_time: str = "",
        count: int = -1,
        dividend_type: str = "none",
        **params,
    ) -> dict:
        """Call a formula/indicator on multiple stocks."""
        return self._post("/api/formula/call_batch", {
            "formula_name": formula_name,
            "stock_codes": stock_codes,
            "period": period,
            "start_time": start_time,
            "end_time": end_time,
            "count": count,
            "dividend_type": dividend_type,
            "params": params,
        })

    def generate_index_data(
        self,
        index_code: str,
        stocks: list[str],
        weights: list[float],
        period: str = "1d",
        start_time: str = "",
        end_time: str = "",
    ) -> dict:
        """Generate custom index data from stocks and weights."""
        return self._post("/api/formula/generate_index", {
            "index_code": index_code,
            "stocks": stocks,
            "weights": weights,
            "period": period,
            "start_time": start_time,
            "end_time": end_time,
        })



"""Router — SMT (约定式交易) endpoints /api/smt/* (requires API Key)."""

from fastapi import APIRouter, Depends

from ..deps import get_trader_manager
from ..helpers import _numpy_to_python
from ..models import SMTNegotiateOrderRequest, SMTOrderRequest, SMTQueryRequest
from ..security import require_api_key

router = APIRouter(prefix="/api/smt", tags=["smt"], dependencies=[Depends(require_api_key)])


@router.post("/order")
def smt_order(req: SMTOrderRequest, manager=Depends(get_trader_manager)):
    """Place an SMT order."""
    result = manager.smt_order(
        stock_code=req.stock_code,
        order_type=req.order_type,
        order_volume=req.order_volume,
        price_type=req.price_type,
        price=req.price,
        smt_type=req.smt_type,
        strategy_name=req.strategy_name,
        order_remark=req.order_remark,
        account_id=req.account_id,
    )
    return {"order_id": result, "status": "submitted"}


@router.post("/negotiate_order_async")
def smt_negotiate_order_async(req: SMTNegotiateOrderRequest, manager=Depends(get_trader_manager)):
    """Place an SMT negotiate order asynchronously (result via WebSocket callback)."""
    result = manager.smt_negotiate_order_async(
        stock_code=req.stock_code,
        order_type=req.order_type,
        order_volume=req.order_volume,
        price=req.price,
        compact_id=req.compact_id,
        strategy_name=req.strategy_name,
        order_remark=req.order_remark,
        account_id=req.account_id,
    )
    return {"seq": result, "status": "async_submitted"}


@router.post("/cancel")
def cancel_smt_order(order_id: int, account_id: str = "", manager=Depends(get_trader_manager)):
    """Cancel an SMT order."""
    result = manager.cancel_smt_order(order_id=order_id, account_id=account_id)
    return {"status": "ok", "data": _numpy_to_python(result)}


@router.get("/quoter")
def smt_query_quoter(
    account_id: str = "",
    manager=Depends(get_trader_manager),
):
    """Query SMT quoter information (报价方信息)."""
    result = manager.smt_query_quoter(account_id=account_id)
    return {"data": _numpy_to_python(result)}


@router.get("/compact")
def smt_query_compact(
    account_id: str = "",
    manager=Depends(get_trader_manager),
):
    """Query SMT compacts (约定合约)."""
    result = manager.smt_query_compact(account_id=account_id)
    return {"data": _numpy_to_python(result)}


@router.get("/appointment")
def query_appointment_info(
    account_id: str = "",
    manager=Depends(get_trader_manager),
):
    """Query SMT appointment info (约定式预约信息)."""
    result = manager.query_appointment_info(account_id=account_id)
    return {"data": _numpy_to_python(result)}


@router.get("/secu_info")
def query_smt_secu_info(
    account_id: str = "",
    manager=Depends(get_trader_manager),
):
    """Query SMT security info (约定式证券信息)."""
    result = manager.query_smt_secu_info(account_id=account_id)
    return {"data": _numpy_to_python(result)}


@router.get("/secu_rate")
def query_smt_secu_rate(
    account_id: str = "",
    manager=Depends(get_trader_manager),
):
    """Query SMT security rates (约定式证券费率)."""
    result = manager.query_smt_secu_rate(account_id=account_id)
    return {"data": _numpy_to_python(result)}



"""Format trading events into Feishu interactive card messages."""

from __future__ import annotations

# Header color mapping (Feishu template_color)
_COLORS: dict[str, str] = {
    "trade": "green",
    "order": "blue",
    "order_error": "red",
    "cancel_error": "red",
    "connected": "green",
    "disconnected": "red",
    "asset": "blue",
    "position": "blue",
    "account_status": "blue",
    "test": "turquoise",
}

_TITLES: dict[str, str] = {
    "trade": "成交通知",
    "order": "委托更新",
    "order_error": "委托错误",
    "cancel_error": "撤单失败",
    "connected": "交易连接",
    "disconnected": "连接断开",
    "asset": "资产变动",
    "position": "持仓变动",
    "account_status": "账户状态",
    "test": "测试通知",
}

# order_type mapping
_ORDER_TYPE_MAP: dict[int, str] = {
    23: "买入",
    24: "卖出",
}


def _field(label: str, value: object) -> dict:
    """Build a single Feishu card field element."""
    return {
        "is_short": True,
        "text": {
            "tag": "lark_md",
            "content": f"**{label}：**{value}",
        },
    }


def _build_fields(event: dict) -> list[dict]:
    """Build card fields list based on event type."""
    etype = event.get("type", "")
    data: dict = event.get("data", {})

    if etype == "trade":
        direction = _ORDER_TYPE_MAP.get(data.get("order_type"), str(data.get("order_type", "")))
        amount = (data.get("traded_volume", 0) or 0) * (data.get("traded_price", 0) or 0)
        return [
            _field("股票", data.get("stock_code", "")),
            _field("方向", direction),
            _field("成交量", data.get("traded_volume", "")),
            _field("成交价", data.get("traded_price", "")),
            _field("成交金额", f"{amount:.2f}"),
            _field("委托编号", data.get("order_id", "")),
        ]

    if etype == "order":
        direction = _ORDER_TYPE_MAP.get(data.get("order_type"), str(data.get("order_type", "")))
        return [
            _field("股票", data.get("stock_code", "")),
            _field("方向", direction),
            _field("委托量", data.get("order_volume", "")),
            _field("委托价", data.get("price", "")),
            _field("已成交", data.get("traded_volume", "")),
            _field("状态", data.get("status_msg", data.get("order_status", ""))),
        ]

    if etype in ("order_error", "cancel_error"):
        return [
            _field("委托编号", data.get("order_id", "")),
            _field("错误代码", data.get("error_id", "")),
            _field("错误消息", data.get("error_msg", "")),
        ]

    if etype in ("connected", "disconnected"):
        return [_field("状态", "已连接" if etype == "connected" else "已断开")]

    if etype == "asset":
        return [
            _field("总资产", data.get("total_asset", "")),
            _field("可用资金", data.get("cash", "")),
            _field("冻结资金", data.get("frozen_cash", "")),
            _field("持仓市值", data.get("market_value", "")),
        ]

    if etype == "position":
        return [
            _field("股票", data.get("stock_code", "")),
            _field("持仓", data.get("volume", "")),
            _field("可用", data.get("can_use_volume", "")),
            _field("市值", data.get("market_value", "")),
        ]

    if etype == "account_status":
        return [_field("状态", data.get("status", ""))]

    if etype == "test":
        return [_field("消息", data.get("message", ""))]

    # Fallback: render all data fields
    return [_field(k, v) for k, v in data.items()] if data else []


def format_feishu_card(event: dict) -> dict:
    """Build a complete Feishu interactive card message body."""
    etype = event.get("type", "unknown")
    title = _TITLES.get(etype, f"通知 ({etype})")
    color = _COLORS.get(etype, "blue")
    fields = _build_fields(event)

    return {
        "msg_type": "interactive",
        "card": {
            "header": {
                "title": {"tag": "plain_text", "content": f"📌 {title}"},
                "template": color,
            },
            "elements": [
                {
                    "tag": "div",
                    "fields": fields,
                },
                {
                    "tag": "note",
                    "elements": [
                        {
                            "tag": "plain_text",
                            "content": "QMT Bridge",
                        }
                    ],
                },
            ],
        },
    }



"""CreditMixin — credit (margin) trading client methods."""


class CreditMixin:
    """Client methods for /api/credit/* endpoints."""

    def credit_order(
        self,
        stock_code: str,
        order_type: int,
        order_volume: int,
        price_type: int = 5,
        price: float = 0.0,
        credit_type: str = "fin_buy",
        strategy_name: str = "",
        order_remark: str = "",
        account_id: str = "",
    ) -> dict:
        """Place a credit (margin) trading order."""
        return self._post("/api/credit/order", {
            "stock_code": stock_code,
            "order_type": order_type,
            "order_volume": order_volume,
            "price_type": price_type,
            "price": price,
            "credit_type": credit_type,
            "strategy_name": strategy_name,
            "order_remark": order_remark,
            "account_id": account_id,
        })

    def query_credit_positions(self, account_id: str = "") -> dict:
        """Query credit trading positions."""
        return self._get("/api/credit/positions", {"account_id": account_id})

    def query_credit_asset(self, account_id: str = "") -> dict:
        """Query credit trading account asset."""
        return self._get("/api/credit/asset", {"account_id": account_id})

    def query_credit_debt(self, account_id: str = "") -> dict:
        """Query credit debt information."""
        return self._get("/api/credit/debt", {"account_id": account_id})

    def query_credit_available(self, stock_code: str = "", account_id: str = "") -> dict:
        """Query available credit amount for a stock."""
        return self._get("/api/credit/available_amount", {
            "stock_code": stock_code,
            "account_id": account_id,
        })

    def query_slo_stocks(self, account_id: str = "") -> dict:
        """Query stocks available for short selling."""
        return self._get("/api/credit/slo_stocks", {"account_id": account_id})

    def query_fin_stocks(self, account_id: str = "") -> dict:
        """Query stocks available for margin buying."""
        return self._get("/api/credit/fin_stocks", {"account_id": account_id})

    def query_credit_subjects(self, account_id: str = "") -> dict:
        """Query credit subject list (标的证券)."""
        return self._get("/api/credit/subjects", {"account_id": account_id})

    def query_credit_assure(self, account_id: str = "") -> dict:
        """Query credit assurance / collateral info (担保品)."""
        return self._get("/api/credit/assure", {"account_id": account_id})
